@@ -1,32 +1,37 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import six
-from six.moves import xrange
-from six import unichr
+from collections import OrderedDict
 
-import os, base64, tempfile, gzip, io, sys, codecs, re
+import six
+from six import unichr
+from six.moves import xrange
+
+import base64
+import codecs
+import gzip
+import hashlib
+import io
+import logging
+import re
+import uuid
 
 import numpy as np
 
-from hashlib import md5
-
-from matplotlib import verbose, __version__, rcParams
-from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
-     FigureManagerBase, FigureCanvasBase
+from matplotlib import cbook, __version__, rcParams
+from matplotlib.backend_bases import (
+     _Backend, FigureCanvasBase, FigureManagerBase, RendererBase)
 from matplotlib.backends.backend_mixed import MixedModeRenderer
-from matplotlib.cbook import is_string_like, is_writable_file_like, maxdict
 from matplotlib.colors import rgb2hex
-from matplotlib.figure import Figure
-from matplotlib.font_manager import findfont, FontProperties
-from matplotlib.ft2font import FT2Font, KERNING_DEFAULT, LOAD_NO_HINTING
+from matplotlib.font_manager import findfont, get_font
+from matplotlib.ft2font import LOAD_NO_HINTING
 from matplotlib.mathtext import MathTextParser
 from matplotlib.path import Path
 from matplotlib import _path
 from matplotlib.transforms import Affine2D, Affine2DBase
 from matplotlib import _png
 
-from xml.sax.saxutils import escape as escape_xml_text
+_log = logging.getLogger(__name__)
 
 backend_version = __version__
 
@@ -86,13 +91,20 @@ def escape_attrib(s):
     s = s.replace(">", "&gt;")
     return s
 
+def short_float_fmt(x):
+    """
+    Create a short string representation of a float, which is %f
+    formatting with trailing zeros and the decimal point removed.
+    """
+    return '{0:f}'.format(x).rstrip('0').rstrip('.')
+
 ##
 # XML writer class.
 #
 # @param file A file or file-like object.  This object must implement
 #    a <b>write</b> method that takes an 8-bit string.
 
-class XMLWriter:
+class XMLWriter(object):
     def __init__(self, file):
         self.__write = file.write
         if hasattr(file, "flush"):
@@ -135,8 +147,7 @@ class XMLWriter:
         if attrib or extra:
             attrib = attrib.copy()
             attrib.update(extra)
-            attrib = list(six.iteritems(attrib))
-            attrib.sort()
+            attrib = sorted(six.iteritems(attrib))
             for k, v in attrib:
                 if not v == '':
                     k = escape_cdata(k)
@@ -230,15 +241,15 @@ def generate_transform(transform_list=[]):
             if type == 'matrix' and isinstance(value, Affine2DBase):
                 value = value.to_values()
 
-            output.write('%s(%s)' % (type, ' '.join(str(x) for x in value)))
+            output.write('%s(%s)' % (
+                type, ' '.join(short_float_fmt(x) for x in value)))
         return output.getvalue()
     return ''
 
 def generate_css(attrib={}):
     if attrib:
         output = io.StringIO()
-        attrib = list(six.iteritems(attrib))
-        attrib.sort()
+        attrib = sorted(six.iteritems(attrib))
         for k, v in attrib:
             k = escape_attrib(k)
             v = escape_attrib(v)
@@ -248,39 +259,38 @@ def generate_css(attrib={}):
 
 _capstyle_d = {'projecting' : 'square', 'butt' : 'butt', 'round': 'round',}
 class RendererSVG(RendererBase):
-    FONT_SCALE = 100.0
-    fontd = maxdict(50)
-
     def __init__(self, width, height, svgwriter, basename=None, image_dpi=72):
         self.width = width
         self.height = height
         self.writer = XMLWriter(svgwriter)
-        self.image_dpi = image_dpi # the actual dpi we want to rasterize stuff with
+        self.image_dpi = image_dpi  # the actual dpi we want to rasterize stuff with
 
         self._groupd = {}
         if not rcParams['svg.image_inline']:
             assert basename is not None
             self.basename = basename
             self._imaged = {}
-        self._clipd = {}
+        self._clipd = OrderedDict()
         self._char_defs = {}
         self._markers = {}
         self._path_collection_id = 0
         self._imaged = {}
-        self._hatchd = {}
+        self._hatchd = OrderedDict()
         self._has_gouraud = False
         self._n_gradients = 0
-        self._fonts = {}
+        self._fonts = OrderedDict()
         self.mathtext_parser = MathTextParser('SVG')
 
         RendererBase.__init__(self)
         self._glyph_map = dict()
-
+        str_height = short_float_fmt(height)
+        str_width = short_float_fmt(width)
         svgwriter.write(svgProlog)
         self._start_id = self.writer.start(
             'svg',
-            width='%ipt' % width, height='%ipt' % height,
-            viewBox='0 0 %i %i' % (width, height),
+            width='%spt' % str_width,
+            height='%spt' % str_height,
+            viewBox='0 0 %s %s' % (str_width, str_height),
             xmlns="http://www.w3.org/2000/svg",
             version="1.1",
             attrib={'xmlns:xlink': "http://www.w3.org/1999/xlink"})
@@ -306,9 +316,17 @@ class RendererSVG(RendererBase):
 
     def _make_id(self, type, content):
         content = str(content)
+        if rcParams['svg.hashsalt'] is None:
+            salt = str(uuid.uuid4())
+        else:
+            salt = rcParams['svg.hashsalt']
         if six.PY3:
             content = content.encode('utf8')
-        return '%s%s' % (type, md5(content).hexdigest()[:10])
+            salt = salt.encode('utf8')
+        m = hashlib.md5()
+        m.update(salt)
+        m.update(content)
+        return '%s%s' % (type, m.hexdigest()[:10])
 
     def _make_flip_transform(self, transform):
         return (transform +
@@ -317,15 +335,8 @@ class RendererSVG(RendererBase):
                 .translate(0.0, self.height))
 
     def _get_font(self, prop):
-        key = hash(prop)
-        font = self.fontd.get(key)
-        if font is None:
-            fname = findfont(prop)
-            font = self.fontd.get(fname)
-            if font is None:
-                font = FT2Font(fname)
-                self.fontd[fname] = font
-            self.fontd[key] = font
+        fname = findfont(prop)
+        font = get_font(fname)
         font.clear()
         size = prop.get_size_in_points()
         font.set_size(size, 72.0)
@@ -337,7 +348,7 @@ class RendererSVG(RendererBase):
         """
         if rgbFace is not None:
             rgbFace = tuple(rgbFace)
-        edge = gc.get_rgb()
+        edge = gc.get_hatch_color()
         if edge is not None:
             edge = tuple(edge)
         dictkey = (gc.get_hatch(), rgbFace, edge)
@@ -381,7 +392,7 @@ class RendererSVG(RendererBase):
                 style=generate_css({
                     'fill': rgb2hex(stroke),
                     'stroke': rgb2hex(stroke),
-                    'stroke-width': '1.0',
+                    'stroke-width': six.text_type(rcParams['hatch.linewidth']),
                     'stroke-linecap': 'butt',
                     'stroke-linejoin': 'miter'
                     })
@@ -401,7 +412,7 @@ class RendererSVG(RendererBase):
         if gc.get_hatch() is not None:
             attrib['fill'] = "url(#%s)" % self._get_hatch(gc, rgbFace)
             if rgbFace is not None and len(rgbFace) == 4 and rgbFace[3] != 1.0 and not forced_alpha:
-                attrib['fill-opacity'] = str(rgbFace[3])
+                attrib['fill-opacity'] = short_float_fmt(rgbFace[3])
         else:
             if rgbFace is None:
                 attrib['fill'] = 'none'
@@ -409,24 +420,24 @@ class RendererSVG(RendererBase):
                 if tuple(rgbFace[:3]) != (0, 0, 0):
                     attrib['fill'] = rgb2hex(rgbFace)
                 if len(rgbFace) == 4 and rgbFace[3] != 1.0 and not forced_alpha:
-                    attrib['fill-opacity'] = str(rgbFace[3])
+                    attrib['fill-opacity'] = short_float_fmt(rgbFace[3])
 
         if forced_alpha and gc.get_alpha() != 1.0:
-            attrib['opacity'] = str(gc.get_alpha())
+            attrib['opacity'] = short_float_fmt(gc.get_alpha())
 
         offset, seq = gc.get_dashes()
         if seq is not None:
-            attrib['stroke-dasharray'] = ','.join(['%f' % val for val in seq])
-            attrib['stroke-dashoffset'] = six.text_type(float(offset))
+            attrib['stroke-dasharray'] = ','.join([short_float_fmt(val) for val in seq])
+            attrib['stroke-dashoffset'] = short_float_fmt(float(offset))
 
         linewidth = gc.get_linewidth()
         if linewidth:
             rgb = gc.get_rgb()
             attrib['stroke'] = rgb2hex(rgb)
             if not forced_alpha and rgb[3] != 1.0:
-                attrib['stroke-opacity'] = str(rgb[3])
+                attrib['stroke-opacity'] = short_float_fmt(rgb[3])
             if linewidth != 1.0:
-                attrib['stroke-width'] = str(linewidth)
+                attrib['stroke-width'] = short_float_fmt(linewidth)
             if gc.get_joinstyle() != 'round':
                 attrib['stroke-linejoin'] = gc.get_joinstyle()
             if gc.get_capstyle() != 'butt':
@@ -474,8 +485,12 @@ class RendererSVG(RendererBase):
                 writer.element('path', d=path_data)
             else:
                 x, y, w, h = clip
-                writer.element('rect', x=six.text_type(x), y=six.text_type(y),
-                               width=six.text_type(w), height=six.text_type(h))
+                writer.element(
+                    'rect',
+                    x=short_float_fmt(x),
+                    y=short_float_fmt(y),
+                    width=short_float_fmt(w),
+                    height=short_float_fmt(h))
             writer.end('clipPath')
         writer.end('defs')
 
@@ -486,17 +501,18 @@ class RendererSVG(RendererBase):
         writer = self.writer
         writer.start('defs')
         for font_fname, chars in six.iteritems(self._fonts):
-            font = FT2Font(font_fname)
+            font = get_font(font_fname)
             font.set_size(72, 72)
             sfnt = font.get_sfnt()
-            writer.start('font', id=sfnt[(1, 0, 0, 4)])
+            writer.start('font', id=sfnt[1, 0, 0, 4].decode("mac_roman"))
             writer.element(
                 'font-face',
                 attrib={
                     'font-family': font.family_name,
                     'font-style': font.style_name.lower(),
                     'units-per-em': '72',
-                    'bbox': ' '.join(six.text_type(x / 64.0) for x in font.bbox)})
+                    'bbox': ' '.join(
+                        short_float_fmt(x / 64.0) for x in font.bbox)})
             for char in chars:
                 glyph = font.load_char(char, flags=LOAD_NO_HINTING)
                 verts, codes = font.get_path()
@@ -509,7 +525,8 @@ class RendererSVG(RendererBase):
                     attrib={
                         # 'glyph-name': name,
                         'unicode': unichr(char),
-                        'horiz-adv-x': six.text_type(glyph.linearHoriAdvance / 65536.0)})
+                        'horiz-adv-x':
+                        short_float_fmt(glyph.linearHoriAdvance / 65536.0)})
             writer.end('font')
         writer.end('defs')
 
@@ -529,23 +546,28 @@ class RendererSVG(RendererBase):
 
     def option_image_nocomposite(self):
         """
-        if svg.image_noscale is True, compositing multiple images into one is prohibited
+        return whether to generate a composite image from multiple images on
+        a set of axes
         """
-        return rcParams['svg.image_noscale']
+        return not rcParams['image.composite_image']
 
-    def _convert_path(self, path, transform=None, clip=None, simplify=None):
+    def _convert_path(self, path, transform=None, clip=None, simplify=None,
+                      sketch=None):
         if clip:
             clip = (0.0, 0.0, self.width, self.height)
         else:
             clip = None
-        return _path.convert_to_svg(path, transform, clip, simplify, 6)
+        return _path.convert_to_string(
+            path, transform, clip, simplify, sketch, 6,
+            [b'M', b'L', b'Q', b'C', b'z'], False).decode('ascii')
 
     def draw_path(self, gc, path, transform, rgbFace=None):
         trans_and_flip = self._make_flip_transform(transform)
         clip = (rgbFace is None and gc.get_hatch_path() is None)
         simplify = path.should_simplify and clip
         path_data = self._convert_path(
-            path, trans_and_flip, clip=clip, simplify=simplify)
+            path, trans_and_flip, clip=clip, simplify=simplify,
+            sketch=gc.get_sketch_params())
 
         attrib = {}
         attrib['style'] = self._get_style(gc, rgbFace)
@@ -572,10 +594,8 @@ class RendererSVG(RendererBase):
         style = self._get_style_dict(gc, rgbFace)
         dictkey = (path_data, generate_css(style))
         oid = self._markers.get(dictkey)
-        for key in list(six.iterkeys(style)):
-            if not key.startswith('stroke'):
-                del style[key]
-        style = generate_css(style)
+        style = generate_css({k: v for k, v in six.iteritems(style)
+                              if k.startswith('stroke')})
 
         if oid is None:
             oid = self._make_id('m', dictkey)
@@ -597,8 +617,8 @@ class RendererSVG(RendererBase):
                 trans_and_flip, clip=clip, simplify=False):
             if len(vertices):
                 x, y = vertices[-2:]
-                attrib['x'] = six.text_type(x)
-                attrib['y'] = six.text_type(y)
+                attrib['x'] = short_float_fmt(x)
+                attrib['y'] = short_float_fmt(y)
                 attrib['style'] = self._get_style(gc, rgbFace)
                 writer.element('use', attrib=attrib)
         writer.end('g')
@@ -607,6 +627,23 @@ class RendererSVG(RendererBase):
                              offsets, offsetTrans, facecolors, edgecolors,
                              linewidths, linestyles, antialiaseds, urls,
                              offset_position):
+        # Is the optimization worth it? Rough calculation:
+        # cost of emitting a path in-line is
+        #    (len_path + 5) * uses_per_path
+        # cost of definition+use is
+        #    (len_path + 3) + 9 * uses_per_path
+        len_path = len(paths[0].vertices) if len(paths) > 0 else 0
+        uses_per_path = self._iter_collection_uses_per_path(
+            paths, all_transforms, offsets, facecolors, edgecolors)
+        should_do_optimization = \
+            len_path + 9 * uses_per_path + 3 < (len_path + 5) * uses_per_path
+        if not should_do_optimization:
+            return RendererBase.draw_path_collection(
+                self, gc, master_transform, paths, all_transforms,
+                offsets, offsetTrans, facecolors, edgecolors,
+                linewidths, linestyles, antialiaseds, urls,
+                offset_position)
+
         writer = self.writer
         path_codes = []
         writer.start('defs')
@@ -632,8 +669,8 @@ class RendererSVG(RendererBase):
                 writer.start('g', attrib={'clip-path': 'url(#%s)' % clipid})
             attrib = {
                 'xlink:href': '#%s' % path_id,
-                'x': six.text_type(xo),
-                'y': six.text_type(self.height - yo),
+                'x': short_float_fmt(xo),
+                'y': short_float_fmt(self.height - yo),
                 'style': self._get_style(gc0, rgbFace)
                 }
             writer.element('use', attrib=attrib)
@@ -702,13 +739,13 @@ class RendererSVG(RendererBase):
             writer.start(
                 'linearGradient',
                 id="GR%x_%d" % (self._n_gradients, i),
-                x1=six.text_type(x1), y1=six.text_type(y1),
-                x2=six.text_type(xb), y2=six.text_type(yb))
+                x1=short_float_fmt(x1), y1=short_float_fmt(y1),
+                x2=short_float_fmt(xb), y2=short_float_fmt(yb))
             writer.element(
                 'stop',
                 offset='0',
                 style=generate_css({'stop-color': rgb2hex(c),
-                                    'stop-opacity': six.text_type(c[-1])}))
+                                    'stop-opacity': short_float_fmt(c[-1])}))
             writer.element(
                 'stop',
                 offset='1',
@@ -719,7 +756,7 @@ class RendererSVG(RendererBase):
         writer.element(
             'polygon',
             id='GT%x' % self._n_gradients,
-            points=" ".join([six.text_type(x)
+            points=" ".join([short_float_fmt(x)
                              for x in (x1, y1, x2, y2, x3, y3)]))
         writer.end('defs')
 
@@ -729,7 +766,7 @@ class RendererSVG(RendererBase):
             'use',
             attrib={'xlink:href': href,
                     'fill': rgb2hex(avg_color),
-                    'fill-opacity': str(avg_color[-1])})
+                    'fill-opacity': short_float_fmt(avg_color[-1])})
         for i in range(3):
             writer.element(
                 'use',
@@ -761,7 +798,12 @@ class RendererSVG(RendererBase):
     def get_image_magnification(self):
         return self.image_dpi / 72.0
 
-    def draw_image(self, gc, x, y, im, dx=None, dy=None, transform=None):
+    def draw_image(self, gc, x, y, im, transform=None):
+        h, w = im.shape[:2]
+
+        if w == 0 or h == 0:
+            return
+
         attrib = {}
         clipid = self._get_clip(gc)
         if clipid is not None:
@@ -770,81 +812,57 @@ class RendererSVG(RendererBase):
             # to the clip-path
             self.writer.start('g', attrib={'clip-path': 'url(#%s)' % clipid})
 
-        trans = [1,0,0,1,0,0]
-        if rcParams['svg.image_noscale']:
-            trans = list(im.get_matrix())
-            trans[5] = -trans[5]
-            attrib['transform'] = generate_transform([('matrix', tuple(trans))])
-            assert trans[1] == 0
-            assert trans[2] == 0
-            numrows, numcols = im.get_size()
-            im.reset_matrix()
-            im.set_interpolation(0)
-            im.resize(numcols, numrows)
-
-        h,w = im.get_size_out()
-
-        if dx is None:
-            w = 72.0*w/self.image_dpi
-        else:
-            w = dx
-
-        if dy is None:
-            h = 72.0*h/self.image_dpi
-        else:
-            h = dy
-
-        oid = getattr(im, '_gid', None)
-        url = getattr(im, '_url', None)
+        oid = gc.get_gid()
+        url = gc.get_url()
         if url is not None:
             self.writer.start('a', attrib={'xlink:href': url})
         if rcParams['svg.image_inline']:
             bytesio = io.BytesIO()
-            im.flipud_out()
-            rows, cols, buffer = im.as_rgba_str()
-            _png.write_png(buffer, cols, rows, bytesio)
-            im.flipud_out()
-            oid = oid or self._make_id('image', bytesio)
+            _png.write_png(im, bytesio)
+            oid = oid or self._make_id('image', bytesio.getvalue())
             attrib['xlink:href'] = (
                 "data:image/png;base64,\n" +
                 base64.b64encode(bytesio.getvalue()).decode('ascii'))
         else:
-            self._imaged[self.basename] = self._imaged.get(self.basename,0) + 1
+            self._imaged[self.basename] = self._imaged.get(self.basename, 0) + 1
             filename = '%s.image%d.png'%(self.basename, self._imaged[self.basename])
-            verbose.report( 'Writing image file for inclusion: %s' % filename)
-            im.flipud_out()
-            rows, cols, buffer = im.as_rgba_str()
-            _png.write_png(buffer, cols, rows, filename)
-            im.flipud_out()
+            _log.info('Writing image file for inclusion: %s', filename)
+            _png.write_png(im, filename)
             oid = oid or 'Im_' + self._make_id('image', filename)
             attrib['xlink:href'] = filename
 
-        alpha = gc.get_alpha()
-        if alpha != 1.0:
-            attrib['opacity'] = str(alpha)
-
         attrib['id'] = oid
 
         if transform is None:
+            w = 72.0 * w / self.image_dpi
+            h = 72.0 * h / self.image_dpi
+
             self.writer.element(
                 'image',
-                x=six.text_type(x/trans[0]),
-                y=six.text_type((self.height-y)/trans[3]-h),
-                width=six.text_type(w), height=six.text_type(h),
+                transform=generate_transform([
+                    ('scale', (1, -1)), ('translate', (0, -h))]),
+                x=short_float_fmt(x),
+                y=short_float_fmt(-(self.height - y - h)),
+                width=short_float_fmt(w), height=short_float_fmt(h),
                 attrib=attrib)
         else:
-            flipped = self._make_flip_transform(transform)
-            flipped = np.array(flipped.to_values())
-            y = y+dy
-            if dy > 0.0:
-                flipped[3] *= -1.0
-                y *= -1.0
+            alpha = gc.get_alpha()
+            if alpha != 1.0:
+                attrib['opacity'] = short_float_fmt(alpha)
+
+            flipped = (
+                Affine2D().scale(1.0 / w, 1.0 / h) +
+                transform +
+                Affine2D()
+                .translate(x, y)
+                .scale(1.0, -1.0)
+                .translate(0.0, self.height))
+
             attrib['transform'] = generate_transform(
-                [('matrix', flipped)])
+                [('matrix', flipped.frozen())])
             self.writer.element(
                 'image',
-                x=six.text_type(x), y=six.text_type(y),
-                width=six.text_type(dx), height=six.text_type(abs(dy)),
+                width=short_float_fmt(w), height=short_float_fmt(h),
                 attrib=attrib)
 
         if url is not None:
@@ -859,17 +877,20 @@ class RendererSVG(RendererBase):
         """
         draw the text by converting them to paths using textpath module.
 
-        *prop*
+        Parameters
+        ----------
+        prop : `matplotlib.font_manager.FontProperties`
           font property
 
-        *s*
+        s : str
           text to be converted
 
-        *usetex*
+        usetex : bool
           If True, use matplotlib usetex mode.
 
-        *ismath*
+        ismath : bool
           If True, use mathtext parser. If "TeX", use *usetex* mode.
+
         """
         writer = self.writer
 
@@ -885,7 +906,7 @@ class RendererSVG(RendererBase):
         if color != '#000000':
             style['fill'] = color
         if gc.get_alpha() != 1.0:
-            style['opacity'] = six.text_type(gc.get_alpha())
+            style['opacity'] = short_float_fmt(gc.get_alpha())
 
         if not ismath:
             font = text2path._get_font(prop)
@@ -915,9 +936,9 @@ class RendererSVG(RendererBase):
             for glyph_id, xposition, yposition, scale in glyph_info:
                 attrib={'xlink:href': '#%s' % glyph_id}
                 if xposition != 0.0:
-                    attrib['x'] = six.text_type(xposition)
+                    attrib['x'] = short_float_fmt(xposition)
                 if yposition != 0.0:
-                    attrib['y'] = six.text_type(yposition)
+                    attrib['y'] = short_float_fmt(yposition)
                 writer.element(
                     'use',
                     attrib=attrib)
@@ -986,7 +1007,7 @@ class RendererSVG(RendererBase):
         if color != '#000000':
             style['fill'] = color
         if gc.get_alpha() != 1.0:
-            style['opacity'] = six.text_type(gc.get_alpha())
+            style['opacity'] = short_float_fmt(gc.get_alpha())
 
         if not ismath:
             font = self._get_font(prop)
@@ -999,9 +1020,10 @@ class RendererSVG(RendererBase):
 
             attrib = {}
             # Must add "px" to workaround a Firefox bug
-            style['font-size'] = six.text_type(fontsize) + 'px'
+            style['font-size'] = short_float_fmt(fontsize) + 'px'
             style['font-family'] = six.text_type(fontfamily)
             style['font-style'] = prop.get_style().lower()
+            style['font-weight'] = six.text_type(prop.get_weight()).lower()
             attrib['style'] = generate_css(style)
 
             if mtext and (angle == 0 or mtext.get_rotation_mode() == "anchor"):
@@ -1016,7 +1038,7 @@ class RendererSVG(RendererBase):
                 # Don't do vertical anchor alignment. Most applications do not
                 # support 'alignment-baseline' yet. Apply the vertical layout
                 # to the anchor point manually for now.
-                angle_rad = angle * np.pi / 180.
+                angle_rad = np.deg2rad(angle)
                 dir_vert = np.array([np.sin(angle_rad), np.cos(angle_rad)])
                 v_offset = np.dot(dir_vert, [(x - ax), (y - ay)])
                 ax = ax + v_offset * dir_vert[0]
@@ -1026,10 +1048,13 @@ class RendererSVG(RendererBase):
                                  'center': 'middle'}
                 style['text-anchor'] = ha_mpl_to_svg[mtext.get_ha()]
 
-                attrib['x'] = str(ax)
-                attrib['y'] = str(ay)
+                attrib['x'] = short_float_fmt(ax)
+                attrib['y'] = short_float_fmt(ay)
                 attrib['style'] = generate_css(style)
-                attrib['transform'] = "rotate(%f, %f, %f)" % (-angle, ax, ay)
+                attrib['transform'] = "rotate(%s, %s, %s)" % (
+                    short_float_fmt(-angle),
+                    short_float_fmt(ax),
+                    short_float_fmt(ay))
                 writer.element('text', s, attrib=attrib)
             else:
                 attrib['transform'] = generate_transform([
@@ -1065,12 +1090,13 @@ class RendererSVG(RendererBase):
 
             # Sort the characters by font, and output one tspan for
             # each
-            spans = {}
+            spans = OrderedDict()
             for font, fontsize, thetext, new_x, new_y, metrics in svg_glyphs:
                 style = generate_css({
-                    'font-size': six.text_type(fontsize) + 'px',
+                    'font-size': short_float_fmt(fontsize) + 'px',
                     'font-family': font.family_name,
-                    'font-style': font.style_name.lower()})
+                    'font-style': font.style_name.lower(),
+                    'font-weight': font.style_name.lower()})
                 if thetext == 32:
                     thetext = 0xa0 # non-breaking space
                 spans.setdefault(style, []).append((new_x, -new_y, thetext))
@@ -1080,7 +1106,7 @@ class RendererSVG(RendererBase):
                     fontset = self._fonts.setdefault(font.fname, set())
                     fontset.add(thetext)
 
-            for style, chars in list(six.iteritems(spans)):
+            for style, chars in six.iteritems(spans):
                 chars.sort()
 
                 same_y = True
@@ -1097,7 +1123,7 @@ class RendererSVG(RendererBase):
 
                 attrib = {
                     'style': style,
-                    'x': ' '.join(six.text_type(c[0]) for c in chars),
+                    'x': ' '.join(short_float_fmt(c[0]) for c in chars),
                     'y': ys
                     }
 
@@ -1112,8 +1138,10 @@ class RendererSVG(RendererBase):
                 for x, y, width, height in svg_rects:
                     writer.element(
                         'rect',
-                        x=six.text_type(x), y=six.text_type(-y + height),
-                        width=six.text_type(width), height=six.text_type(height)
+                        x=short_float_fmt(x),
+                        y=short_float_fmt(-y + height),
+                        width=short_float_fmt(width),
+                        height=short_float_fmt(height)
                         )
 
             writer.end('g')
@@ -1160,52 +1188,53 @@ class FigureCanvasSVG(FigureCanvasBase):
     fixed_dpi = 72
 
     def print_svg(self, filename, *args, **kwargs):
-        if is_string_like(filename):
-            fh_to_close = svgwriter = io.open(filename, 'w', encoding='utf-8')
-        elif is_writable_file_like(filename):
-            if not isinstance(filename, io.TextIOBase):
-                if six.PY3:
-                    svgwriter = io.TextIOWrapper(filename, 'utf-8')
-                else:
-                    svgwriter = codecs.getwriter('utf-8')(filename)
+        with cbook.open_file_cm(filename, "w", encoding="utf-8") as fh:
+
+            filename = getattr(fh, 'name', '')
+            if not isinstance(filename, six.string_types):
+                filename = ''
+
+            if cbook.file_requires_unicode(fh):
+                detach = False
             else:
-                svgwriter = filename
-            fh_to_close = None
-        else:
-            raise ValueError("filename must be a path or a file-like object")
-        return self._print_svg(filename, svgwriter, fh_to_close, **kwargs)
+                if six.PY3:
+                    fh = io.TextIOWrapper(fh, 'utf-8')
+                else:
+                    fh = codecs.getwriter('utf-8')(fh)
+                detach = True
+
+            result = self._print_svg(filename, fh, **kwargs)
+
+            # Detach underlying stream from wrapper so that it remains open in
+            # the caller.
+            if detach:
+                if six.PY3:
+                    fh.detach()
+                else:
+                    fh.reset()
+                    fh.stream = io.BytesIO()
+
+        return result
 
     def print_svgz(self, filename, *args, **kwargs):
-        if is_string_like(filename):
-            fh_to_close = gzipwriter = gzip.GzipFile(filename, 'w')
-            svgwriter = io.TextIOWrapper(gzipwriter, 'utf-8')
-        elif is_writable_file_like(filename):
-            fh_to_close = gzipwriter = gzip.GzipFile(fileobj=filename, mode='w')
-            svgwriter = io.TextIOWrapper(gzipwriter, 'utf-8')
-        else:
-            raise ValueError("filename must be a path or a file-like object")
-        return self._print_svg(filename, svgwriter, fh_to_close)
+        with cbook.open_file_cm(filename, "wb") as fh, \
+                gzip.GzipFile(mode='w', fileobj=fh) as gzipwriter:
+            return self.print_svg(gzipwriter)
 
-    def _print_svg(self, filename, svgwriter, fh_to_close=None, **kwargs):
-        try:
-            image_dpi = kwargs.pop("dpi", 72)
-            self.figure.set_dpi(72.0)
-            width, height = self.figure.get_size_inches()
-            w, h = width*72, height*72
+    def _print_svg(self, filename, fh, **kwargs):
+        image_dpi = kwargs.pop("dpi", 72)
+        self.figure.set_dpi(72.0)
+        width, height = self.figure.get_size_inches()
+        w, h = width * 72, height * 72
 
-            if rcParams['svg.image_noscale']:
-                renderer = RendererSVG(w, h, svgwriter, filename, image_dpi)
-            else:
-                _bbox_inches_restore = kwargs.pop("bbox_inches_restore", None)
-                renderer = MixedModeRenderer(self.figure,
-                    width, height, image_dpi, RendererSVG(w, h, svgwriter, filename, image_dpi),
-                    bbox_inches_restore=_bbox_inches_restore)
+        _bbox_inches_restore = kwargs.pop("bbox_inches_restore", None)
+        renderer = MixedModeRenderer(
+            self.figure, width, height, image_dpi,
+            RendererSVG(w, h, fh, filename, image_dpi),
+            bbox_inches_restore=_bbox_inches_restore)
 
-            self.figure.draw(renderer)
-            renderer.finalize()
-        finally:
-            if fh_to_close is not None:
-                svgwriter.close()
+        self.figure.draw(renderer)
+        renderer.finalize()
 
     def get_default_filetype(self):
         return 'svg'
@@ -1214,21 +1243,6 @@ class FigureManagerSVG(FigureManagerBase):
     pass
 
 
-def new_figure_manager(num, *args, **kwargs):
-    FigureClass = kwargs.pop('FigureClass', Figure)
-    thisFig = FigureClass(*args, **kwargs)
-    return new_figure_manager_given_figure(num, thisFig)
-
-
-def new_figure_manager_given_figure(num, figure):
-    """
-    Create a new figure manager instance for the given figure.
-    """
-    canvas  = FigureCanvasSVG(figure)
-    manager = FigureManagerSVG(canvas, num)
-    return manager
-
-
 svgProlog = """\
 <?xml version="1.0" encoding="utf-8" standalone="no"?>
 <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"
@@ -1237,5 +1251,7 @@ svgProlog = """\
 """
 
 
-FigureCanvas = FigureCanvasSVG
-FigureManager = FigureManagerSVG
+@_Backend.export
+class _BackendSVG(_Backend):
+    FigureCanvas = FigureCanvasSVG
+    FigureManager = FigureManagerSVG
