@@ -1,12 +1,13 @@
 """
-This module provides a large set of colormaps, functions for
-registering new colormaps and for getting a colormap by name,
-and a mixin class for adding color mapping functionality.
+Builtin colormaps, colormap handling utilities, and the `ScalarMappable` mixin.
 
+See :doc:`/gallery/color/colormap_reference` for a list of builtin colormaps.
+See :doc:`/tutorials/colors/colormaps` for an in-depth discussion of colormaps.
 """
-from __future__ import print_function, division
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
-import os
+import six
 
 import numpy as np
 from numpy import ma
@@ -14,9 +15,11 @@ import matplotlib as mpl
 import matplotlib.colors as colors
 import matplotlib.cbook as cbook
 from matplotlib._cm import datad
-from matplotlib._cm import cubehelix
+from matplotlib._cm_listed import cmaps as cmaps_listed
+
+
+cmap_d = {}
 
-cmap_d = dict()
 
 # reverse all the colormaps.
 # reversed colormaps have '_r' appended to the name.
@@ -31,13 +34,13 @@ def _reverser(f):
 def revcmap(data):
     """Can only handle specification *data* in dictionary format."""
     data_r = {}
-    for key, val in data.iteritems():
+    for key, val in six.iteritems(data):
         if callable(val):
             valnew = _reverser(val)
-                # This doesn't work: lambda x: val(1-x)
-                # The same "val" (the first one) is used
-                # each time, so the colors are identical
-                # and the result is shades of gray.
+            # This doesn't work: lambda x: val(1-x)
+            # The same "val" (the first one) is used
+            # each time, so the colors are identical
+            # and the result is shades of gray.
         else:
             # Flip x and exchange the y values facing x = 0 and x = 1.
             valnew = [(1.0 - x, y1, y0) for x, y0, y1 in reversed(val)]
@@ -49,6 +52,9 @@ def _reverse_cmap_spec(spec):
     """Reverses cmap specification *spec*, can handle both dict and tuple
     type specs."""
 
+    if 'listed' in spec:
+        return {'listed': spec['listed'][::-1]}
+
     if 'red' in spec:
         return revcmap(spec)
     else:
@@ -59,7 +65,7 @@ def _reverse_cmap_spec(spec):
 
 
 def _generate_cmap(name, lutsize):
-    """Generates the requested cmap from it's name *name*.  The lut size is
+    """Generates the requested cmap from its *name*.  The lut size is
     *lutsize*."""
 
     spec = datad[name]
@@ -67,28 +73,28 @@ def _generate_cmap(name, lutsize):
     # Generate the colormap object.
     if 'red' in spec:
         return colors.LinearSegmentedColormap(name, spec, lutsize)
+    elif 'listed' in spec:
+        return colors.ListedColormap(spec['listed'], name)
     else:
         return colors.LinearSegmentedColormap.from_list(name, spec, lutsize)
 
 LUTSIZE = mpl.rcParams['image.lut']
 
-_cmapnames = datad.keys()  # need this list because datad is changed in loop
+# Generate the reversed specifications (all at once, to avoid
+# modify-when-iterating).
+datad.update({cmapname + '_r': _reverse_cmap_spec(spec)
+              for cmapname, spec in six.iteritems(datad)})
 
-# Generate the reversed specifications ...
-
-for cmapname in _cmapnames:
-    spec = datad[cmapname]
-    spec_reversed = _reverse_cmap_spec(spec)
-    datad[cmapname + '_r'] = spec_reversed
-
-# Precache the cmaps with ``lutsize = LUTSIZE`` ...
-
-# Use datad.keys() to also add the reversed ones added in the section above:
-for cmapname in datad.iterkeys():
+# Precache the cmaps with ``lutsize = LUTSIZE``.
+# Also add the reversed ones added in the section above:
+for cmapname in datad:
     cmap_d[cmapname] = _generate_cmap(cmapname, LUTSIZE)
 
+cmap_d.update(cmaps_listed)
+
 locals().update(cmap_d)
 
+
 # Continue with definitions ...
 
 
@@ -117,7 +123,7 @@ def register_cmap(name=None, cmap=None, data=None, lut=None):
         except AttributeError:
             raise ValueError("Arguments must include a name or a Colormap")
 
-    if not cbook.is_string_like(name):
+    if not isinstance(name, six.string_types):
         raise ValueError("Colormap name must be a string")
 
     if isinstance(cmap, colors.Colormap):
@@ -142,9 +148,8 @@ def get_cmap(name=None, lut=None):
     returned.
 
     If *lut* is not None it must be an integer giving the number of
-    entries desired in the lookup table, and *name* must be a
-    standard mpl colormap name with a corresponding data dictionary
-    in *datad*.
+    entries desired in the lookup table, and *name* must be a standard
+    mpl colormap name.
     """
     if name is None:
         name = mpl.rcParams['image.cmap']
@@ -155,13 +160,15 @@ def get_cmap(name=None, lut=None):
     if name in cmap_d:
         if lut is None:
             return cmap_d[name]
-        elif name in datad:
-            return _generate_cmap(name, lut)
-
-    raise ValueError("Colormap %s is not recognized" % name)
+        else:
+            return cmap_d[name]._resample(lut)
+    else:
+        raise ValueError(
+            "Colormap %s is not recognized. Possible values are: %s"
+            % (name, ', '.join(sorted(cmap_d))))
 
 
-class ScalarMappable:
+class ScalarMappable(object):
     """
     This is a mixin class to support scalar data to RGBA mapping.
     The ScalarMappable makes use of data normalization before returning
@@ -176,9 +183,10 @@ class ScalarMappable:
         norm : :class:`matplotlib.colors.Normalize` instance
             The normalizing object which scales data, typically into the
             interval ``[0, 1]``.
+            If *None*, *norm* defaults to a *colors.Normalize* object which
+            initializes its scaling based on the first data processed.
         cmap : str or :class:`~matplotlib.colors.Colormap` instance
             The colormap used to map normalized data values to RGBA colors.
-
         """
 
         self.callbacksSM = cbook.CallbackRegistry()
@@ -197,12 +205,7 @@ class ScalarMappable:
         self.colorbar = None
         self.update_dict = {'array': False}
 
-    @cbook.deprecated('1.3', alternative='the colorbar attribute')
-    def set_colorbar(self, im, ax):
-        """set the colorbar and axes instances associated with mappable"""
-        self.colorbar = im
-
-    def to_rgba(self, x, alpha=None, bytes=False):
+    def to_rgba(self, x, alpha=None, bytes=False, norm=True):
         """
         Return a normalized rgba array corresponding to *x*.
 
@@ -215,6 +218,9 @@ class ScalarMappable:
         If *x* is an ndarray with 3 dimensions,
         and the last dimension is either 3 or 4, then it will be
         treated as an rgb or rgba array, and no mapping will be done.
+        The array can be uint8, or it can be floating point with
+        values in the 0-1 range; otherwise a ValueError will be raised.
+        If it is a masked array, the mask will be ignored.
         If the last dimension is 3, the *alpha* kwarg (defaulting to 1)
         will be used to fill in the transparency.  If the last dimension
         is 4, the *alpha* kwarg is ignored; it does not
@@ -225,10 +231,9 @@ class ScalarMappable:
         array will be floats in the 0-1 range; if it is *True*,
         the returned rgba array will be uint8 in the 0 to 255 range.
 
-        Note: this method assumes the input is well-behaved; it does
-        not check for anomalies such as *x* being a masked rgba
-        array, or being an integer type other than uint8, or being
-        a floating point rgba array with values outside the 0-1 range.
+        If norm is False, no normalization of the input data is
+        performed, and it is assumed to be in the range (0-1).
+
         """
         # First check for special case, image input:
         try:
@@ -246,10 +251,18 @@ class ScalarMappable:
                     xx = x
                 else:
                     raise ValueError("third dimension must be 3 or 4")
-                if bytes and xx.dtype != np.uint8:
-                    xx = (xx * 255).astype(np.uint8)
-                if not bytes and xx.dtype == np.uint8:
-                    xx = xx.astype(float) / 255
+                if xx.dtype.kind == 'f':
+                    if norm and xx.max() > 1 or xx.min() < 0:
+                        raise ValueError("Floating point image RGB values "
+                                         "must be in the 0..1 range.")
+                    if bytes:
+                        xx = (xx * 255).astype(np.uint8)
+                elif xx.dtype == np.uint8:
+                    if not bytes:
+                        xx = xx.astype(np.float32) / 255
+                else:
+                    raise ValueError("Image RGB array must be uint8 or "
+                                     "floating point; found %s" % xx.dtype)
                 return xx
         except AttributeError:
             # e.g., x is not an ndarray; so try mapping it
@@ -257,12 +270,20 @@ class ScalarMappable:
 
         # This is the normal case, mapping a scalar array:
         x = ma.asarray(x)
-        x = self.norm(x)
-        x = self.cmap(x, alpha=alpha, bytes=bytes)
-        return x
+        if norm:
+            x = self.norm(x)
+        rgba = self.cmap(x, alpha=alpha, bytes=bytes)
+        return rgba
 
     def set_array(self, A):
-        'Set the image array from numpy array *A*'
+        """Set the image array from numpy array *A*.
+
+        .. ACCEPTS: ndarray
+
+        Parameters
+        ----------
+        A : ndarray
+        """
         self._A = A
         self.update_dict['array'] = True
 
@@ -284,12 +305,14 @@ class ScalarMappable:
         sequence, interpret it as ``(vmin, vmax)`` which is used to
         support setp
 
-        ACCEPTS: a length 2 sequence of floats
+        ACCEPTS: a length 2 sequence of floats; may be overridden in methods
+        that have ``vmin`` and ``vmax`` kwargs.
         """
-        if (vmin is not None and vmax is None and
-                cbook.iterable(vmin) and len(vmin) == 2):
-            vmin, vmax = vmin
-
+        if vmax is None:
+            try:
+                vmin, vmax = vmin
+            except (TypeError, ValueError):
+                pass
         if vmin is not None:
             self.norm.vmin = vmin
         if vmax is not None:
@@ -307,7 +330,14 @@ class ScalarMappable:
         self.changed()
 
     def set_norm(self, norm):
-        'set the normalization instance'
+        """Set the normalization instance.
+
+        .. ACCEPTS: `.Normalize`
+
+        Parameters
+        ----------
+        norm : `.Normalize`
+        """
         if norm is None:
             norm = colors.Normalize()
         self.norm = norm
@@ -359,3 +389,4 @@ class ScalarMappable:
 
         for key in self.update_dict:
             self.update_dict[key] = True
+        self.stale = True
