@@ -179,7 +179,7 @@ def test_cycle_reset():
     assert prop != next(ax._get_lines.prop_cycler)
     ax.set_prop_cycle(None)
     got = next(ax._get_lines.prop_cycler)
-    assert prop == got, "expected %s, got %s" % (prop, got)
+    assert prop == got
 
     fig, ax = plt.subplots()
     # Need to double-check the old set/get_color_cycle(), too
@@ -190,7 +190,7 @@ def test_cycle_reset():
         assert prop != next(ax._get_lines.prop_cycler)
         ax.set_color_cycle(None)
         got = next(ax._get_lines.prop_cycler)
-        assert prop == got, "expected %s, got %s" % (prop, got)
+        assert prop == got
 
 
 def test_invalid_input_forms():
