@@ -1,5 +1,8 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function
+
+import os
+import sys
+import warnings
 
 from matplotlib import rcParams
 from matplotlib.testing.decorators import image_comparison
@@ -7,11 +10,52 @@ from matplotlib.axes import Axes
 from matplotlib.ticker import AutoMinorLocator, FixedFormatter
 import matplotlib.pyplot as plt
 import matplotlib.dates as mdates
+import matplotlib.gridspec as gridspec
 import numpy as np
-import warnings
 import pytest
 
 
+@image_comparison(baseline_images=['figure_align_labels'])
+def test_align_labels():
+    # Check the figure.align_labels() command
+    fig = plt.figure(tight_layout=True)
+    gs = gridspec.GridSpec(3, 3)
+
+    ax = fig.add_subplot(gs[0, :2])
+    ax.plot(np.arange(0, 1e6, 1000))
+    ax.set_ylabel('Ylabel0 0')
+    ax = fig.add_subplot(gs[0, -1])
+    ax.plot(np.arange(0, 1e4, 100))
+
+    for i in range(3):
+        ax = fig.add_subplot(gs[1, i])
+        ax.set_ylabel('YLabel1 %d' % i)
+        ax.set_xlabel('XLabel1 %d' % i)
+        if i in [0, 2]:
+            ax.xaxis.set_label_position("top")
+            ax.xaxis.tick_top()
+        if i == 0:
+            for tick in ax.get_xticklabels():
+                tick.set_rotation(90)
+        if i == 2:
+            ax.yaxis.set_label_position("right")
+            ax.yaxis.tick_right()
+
+    for i in range(3):
+        ax = fig.add_subplot(gs[2, i])
+        ax.set_xlabel('XLabel2 %d' % (i))
+        ax.set_ylabel('YLabel2 %d' % (i))
+
+        if i == 2:
+            ax.plot(np.arange(0, 1e4, 10))
+            ax.yaxis.set_label_position("right")
+            ax.yaxis.tick_right()
+            for tick in ax.get_xticklabels():
+                tick.set_rotation(90)
+
+    fig.align_labels()
+
+
 def test_figure_label():
     # pyplot figure creation, selection and closing with figure label and
     # number
@@ -322,7 +366,19 @@ def test_subplots_shareax_loglabels():
 
 
 def test_savefig():
-    fig, ax = plt.subplots()
+    fig = plt.figure()
     msg = "savefig() takes 2 positional arguments but 3 were given"
     with pytest.raises(TypeError, message=msg):
         fig.savefig("fname1.png", "fname2.png")
+
+
+def test_figure_repr():
+    fig = plt.figure(figsize=(10, 20), dpi=10)
+    assert repr(fig) == "<Figure size 100x200 with 0 Axes>"
+
+
+@pytest.mark.skipif(sys.version_info < (3, 6), reason="requires Python 3.6+")
+@pytest.mark.parametrize("fmt", ["png", "pdf", "ps", "eps", "svg"])
+def test_fspath(fmt):
+    from pathlib import Path
+    plt.savefig(Path(os.devnull), format=fmt)
