@@ -1,5 +1,4 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function
 
 import six
 
@@ -7,7 +6,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 from matplotlib.testing.decorators import image_comparison
 
-from matplotlib.table import CustomCell
+from matplotlib.table import CustomCell, Table
 from matplotlib.path import Path
 
 
@@ -182,3 +181,16 @@ def test_auto_column():
     tb4.auto_set_font_size(False)
     tb4.set_fontsize(12)
     tb4.auto_set_column_width("-101")
+
+
+def test_table_cells():
+    fig, ax = plt.subplots()
+    table = Table(ax)
+
+    cell = table.add_cell(1, 2, 1, 1)
+    assert isinstance(cell, CustomCell)
+    assert cell is table[1, 2]
+
+    cell2 = CustomCell((0, 0), 1, 2, visible_edges=None)
+    table[2, 1] = cell2
+    assert table[2, 1] is cell2
